MODULE dom_oce
   !!----------------------------------------------------------------------
   !!                       ***  MODULE dom_oce  ***
   !!       
   !! ** Purpose :   Define in memory all the ocean space domain variables
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL  (2005)
   !!   $Id$
   !!   This software is governed by the CeCILL licence see modipsl/doc/NEMO_CeCILL.txt
   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_oce      ! ocean parameters

   IMPLICIT NONE
   PUBLIC           ! allows the acces to par_oce when dom_oce is used
   !                ! exception to coding rules... to be suppressed ???

   !!----------------------------------------------------------------------
   !! space domain parameters
   !! -----------------------
   LOGICAL, PUBLIC ::   &   !:
      lzoom      =  .FALSE. ,   &  !: zoom flag
      lzoom_e    =  .FALSE. ,   &  !: East  zoom type flag
      lzoom_w    =  .FALSE. ,   &  !: West  zoom type flag
      lzoom_s    =  .FALSE. ,   &  !: South zoom type flag
      lzoom_n    =  .FALSE. ,   &  !: North zoom type flag
      lzoom_arct =  .FALSE. ,   &  !: ORCA    arctic zoom flag
      lzoom_anta =  .FALSE.        !: ORCA antarctic zoom flag

   INTEGER, PUBLIC ::           & !!: namdom : space domain (bathymetry, mesh)
      nmsh    =  0                 !: = 1 create a mesh-mask file

   INTEGER, PUBLIC ::         &   !:
      ! domain parameters linked to mpp
      nperio,          &  !: type of lateral boundary condition
      nimpp, njmpp,    &  !: i- & j-indexes for mpp-subdomain left bottom
      nreci, nrecj,    &  !: overlap region in i and j
      nproc,           &  !: number for local processor
      narea,           &  !: number for local area
      nbondi, nbondj,  &  !: mark of i- and j-direction local boundaries
      npolj,           &  !: north fold mark (0, 3 or 4)
      nlci, nlcj,      &  !: i- & j-dimensions of the local subdomain
      nldi, nlei,      &  !: first and last indoor i- and j-indexes
      nldj, nlej,      &  !:
      noea, nowe,      &  !: index of the local neighboring processors in
      noso, nono,      &  !: east, west, south and north directions
      npne, npnw,      &  !: index of north east and north west processor
      npse, npsw,      &  !: index of south east and south west processor
      nbne, nbnw,      &  !: logical of north east & north west processor
      nbse, nbsw,      &  !: logical of south east & south west processor
      nidom

   INTEGER, PUBLIC, DIMENSION(jpi) ::   &   !:
      mig                 !: local  ==> global  domain i-indice
   INTEGER, PUBLIC, DIMENSION(jpj) ::   &   !:
      mjg                 !: local  ==> global  domain j-indice
   INTEGER, PUBLIC, DIMENSION( jpidta ) ::   &  !:  !!bug ==> other solution?
      mi0, mi1            !: global ==> local domain i-indice
      !                   !  (mi0=1 and mi1=0 if the global indice is not in the local domain)
   INTEGER, PUBLIC, DIMENSION( jpjdta ) ::   &  !:
      mj0, mj1            !: global ==> local domain j-indice
      !                   ! (mi0=1 and mi1=0 if the global indice is not in the local domain)

   INTEGER, PUBLIC, DIMENSION(jpnij) ::   &  !:
      nimppt, njmppt,  &  !: i-, j-indexes for each processor
      ibonit, ibonjt,  &  !: i-, j- processor neighbour existence
      nlcit, nlcjt,    &  !: dimensions of every subdomain
      nldit, nldjt,    &  !: first, last indoor index for each i-domain
      nleit, nlejt        !: first, last indoor index for each j-domain

   !!----------------------------------------------------------------------
   !! horizontal curvilinear coordinate and scale factors
   !! ---------------------------------------------------------------------

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   &  !:
      glamt, glamu,    &  !: longitude of t-, u-, v- and f-points (degre)
      glamv, glamf,    &  !:
      gphit, gphiu,    &  !: latitude  of t-, u-, v- and f-points (degre)
      gphiv, gphif,    &  !:
      e1t, e2t,        &  !: horizontal scale factors at t-point (m)
      e1u, e2u,        &  !: horizontal scale factors at u-point (m)
      e1v, e2v,        &  !: horizontal scale factors at v-point (m)
      ff                  !: coriolis factor (2.*omega*sin(yphi) ) (s-1)


   !!----------------------------------------------------------------------
   !! vertical coordinate and scale factors
   !! --------------------------------------

   LOGICAL, PUBLIC ::           & !!: namzgr : vertical coordinate
      ln_zco     =  .TRUE.  ,   &  !: z-coordinate - full step
      ln_zps     =  .FALSE. ,   &  !: z-coordinate - partial step
      ln_sco     =  .FALSE.        !: s-coordinate or hybrid z-s coordinate

#if defined key_zco
   LOGICAL, PUBLIC, PARAMETER ::   lk_zco = .TRUE.    !: z-coordinate flag (1D arrays)
#else
   LOGICAL, PUBLIC, PARAMETER ::   lk_zco = .FALSE.   !: z-coordinate flag (3D arrays)

   !! All coordinates
   !! ---------------
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   &  !:
      gdep3w        ,    &  !: depth of T-points (sum of e3w) (m)
      gdept , gdepw ,    &  !: analytical depth at T-W  points (m)
      e3v   , e3f   ,    &  !: analytical vertical scale factors at  V--F
      e3t   , e3u   ,    &  !:                                       T--U  points (m)
      e3vw          ,    &  !: analytical vertical scale factors at  VW--
      e3w   , e3uw          !:                                        W--UW  points (m)
#endif
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   &   !:
      hur, hvr,          &  !: inverse of u and v-points ocean depth (1/m)
      hu , hv               !: depth at u- and v-points (meters)

   !! z-coordinate with full steps (also used in the other cases as reference z-coordinate)
   !! =-----------------====------
   REAL(wp), PUBLIC, DIMENSION(jpk) ::   &  !:
      gdept_0, gdepw_0,       &  !: reference depth of t- and w-points (m)
      e3t_0  , e3w_0             !: reference vertical scale factors at T- and W-pts (m)

   !! z-coordinate with partial steps
   !! =-----------------=======------
   REAL(wp), PUBLIC ::      & !!: * namelist namdom *
      e3zps_min = 5.0_wp,   &  !: miminum thickness for partial steps (meters)
      e3zps_rat = 0.1_wp       !: minimum thickness ration for partial steps

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   &  !:
      hdept, hdepw, e3tp, e3wp   !: ???

   !! s-coordinate and hybrid z-s-coordinate
   !! =----------------======---------------
   REAL(wp), PUBLIC, DIMENSION(jpk) ::   &   !:
      gsigt, gsigw ,   &  !: model level depth coefficient at t-, w-levels (analytic)
      gsi3w        ,   &  !: model level depth coefficient at w-level (sum of gsigw)
      esigt, esigw        !: vertical scale factor coef. at t-, w-levels

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   &   !:
      hbatv , hbatf ,   &  !: ocean depth at the vertical of  V--F
      hbatt , hbatu ,   &  !:                                 T--U  points (m)
      scosrf, scobot,   &  !:  ocean surface and bottom topographies (if deviating from coordinate surfaces in HYBRID)
      hifv  , hiff  ,   &  !: interface depth between stretching    at  V--F
      hift  , hifu         !: and quasi-uniform spacing                 T--U  points (m)

#if defined key_vvl
   LOGICAL, PUBLIC, PARAMETER ::   lk_vvl = .TRUE.    !: variable grid flag
#else
   LOGICAL, PUBLIC, PARAMETER ::   lk_vvl = .FALSE.   !: fixed grid flag
#endif

   !!----------------------------------------------------------------------
   !! masks, bathymetry
   !! -----------------

   INTEGER , PUBLIC, DIMENSION(jpi,jpj) ::   &   !:
      mbathy         !: number of ocean level (=0, 1, ... , jpk-1)

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   &   !:
      tmask_i           !: interior domain T-point mask

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   &   !:
      tmask, umask,    &  !: land/ocean mask at T-, U-, V- and F-points
      vmask, fmask        !:

   REAL(wp), PUBLIC, DIMENSION(jpiglo) ::   &   !:
      tpol                !: north fold mask (nperio= 3 or 4)

   !!----------------------------------------------------------------------
   !! time domain
   !!----------------------------------------------------------------------
   INTEGER, PUBLIC ::    & !!: * Namelist * ???
      nacc   = 0 ,       &  !: = 0/1 use of the acceleration of convergence technique
      neuler                !: restart euler forward option (0=Euler)


   REAL(wp), PUBLIC ::       & !!: * Namelist ??? *
      rdt    = 3600._wp ,    &  !: time step for the dynamics (and tracer if nacc=0)
      rdtmin = 3600._wp ,    &  !: minimum time step on tracers
      rdtmax = 3600._wp ,    &  !: maximum time step on tracers
      rdth   =  800._wp ,    &  !: depth variation of tracer step
      atfp   = 0.1_wp   ,    &  !: asselin time filter parameter
      atfp1                     !: asselin time filter coeff. (atfp1= 1-2*atfp)

   REAL(wp), PUBLIC, DIMENSION(jpk) ::   &  !:
      rdttra                    !: vertical profile of tracer time step

   !!----------------------------------------------------------------------
   !! cross land advection
   !!----------------------------------------------------------------------

   INTEGER, PUBLIC ::       & !!: namelist ???
      n_cla                    !: flag (0/1) for cross land advection to
      !                        ! parameterize exchanges through straits

   !!----------------------------------------------------------------------
   !! calendar variables
   !! ---------------------------------------------------------------------
   INTEGER , PUBLIC ::   nyear       !: current year
   INTEGER , PUBLIC ::   nmonth      !: current month
   INTEGER , PUBLIC ::   nday        !: current day of the month
   INTEGER , PUBLIC ::   ndastp      !: time step date in yyyymmdd format
   INTEGER , PUBLIC ::   nday_year   !: current day counted from jan 1st of the current year
   INTEGER , PUBLIC ::   nsec_year   !: current time step counted in second since 00h jan 1st of the current year
   INTEGER , PUBLIC ::   nsec_month  !: current time step counted in second since 00h 1st day of the current month
   INTEGER , PUBLIC ::   nsec_day    !: current time step counted in second since 00h of the current day
   REAL(wp), PUBLIC ::   fjulday     !: julian day 
   REAL(wp), PUBLIC ::   adatrj      !: number of elapsed days since the begining of the whole simulation
   !                                 !: (cumulative duration of previous runs that may have used different time-step size)
   INTEGER , PUBLIC, DIMENSION(0: 1) ::   nyear_len     !: length in days of the previous/current year 
   INTEGER , PUBLIC, DIMENSION(0:13) ::   nmonth_len    !: length in days of the months of the current year
   INTEGER , PUBLIC, DIMENSION(0:13) ::   nmonth_half   !: second since Jan 1st 0h of the current year and the half of the months
   INTEGER , PUBLIC, DIMENSION(0:13) ::   nmonth_end    !: second since Jan 1st 0h of the current year and the end of the months
   INTEGER , PUBLIC                  ::   nsec1jan000   !: second since Jan 1st 0h of nit000 year and Jan 1st 0h the current year

   !!----------------------------------------------------------------------
   !! agrif domain
   !!----------------------------------------------------------------------
#if defined key_agrif
   LOGICAL, PUBLIC, PARAMETER ::   lk_agrif = .TRUE.    !: agrif flag
#else
   LOGICAL, PUBLIC, PARAMETER ::   lk_agrif = .FALSE.   !: agrif flag
#endif

#if defined key_off_degrad
   !! ------------------------------------------------
   !! Degradation method
   !! --------------------------------------------------
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk) :: &
      facvol  !! volume for degraded regions
#endif

   !!----------------------------------------------------------------------
END MODULE dom_oce
